package com.deltadna.android.sdk.helpers;

import android.app.Application;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;

/**
 * Wrapper around Androids SharedPreferences to implement DeltaDNA player preferences.
 */
public class PlayerPrefs{
	static private final String PREF_BUNDLE = "DELTADNA";
	
	static private Application mApp = null;
	static private SharedPreferences smSharedPrefs = null;
	static private Editor smCurrentEditor = null;
	
	/**
	 * Initialises the prefs, an Application is required to obtain the
	 * SharedPreferences object.
	 * 
	 * @param app The application to set.
	 */
	static public void init(Application app){
		mApp = app;
	}
	/**
	 * Gets the SharedPreferences, will create them if they don't exist.
	 * 
	 * @return The shared preferences object.
	 */
	static private SharedPreferences sharedPrefs(){
		if(smSharedPrefs == null){
			smSharedPrefs = mApp.getSharedPreferences(PREF_BUNDLE, Context.MODE_PRIVATE);
		}
		return smSharedPrefs;
	}
	/**
	 * Gets the current transaction editor, will create one if none exists.
	 * 
	 * @return The current transaction editor.
	 */
	static private Editor currentEditor(){
		if(smCurrentEditor == null){
			smCurrentEditor = sharedPrefs().edit();
		}
		return smCurrentEditor;
	}
	/**
	 * Gets the preference string with the given key, returns defValue if
	 * not found.
	 * 
	 * @param key The key to search for.
	 * @param defValue The default value to use.
	 * 
	 * @return The preference string at key, defValue if not found.
	 */
	static public String getString(String key, String defValue){
		return sharedPrefs().getString(key, defValue);
	}
	/**
	 * Gets the preference integer with the given key, returns defValue if
	 * not found.
	 * 
	 * @param key The key to search for.
	 * @param defValue The default value to use.
	 * 
	 * @return The preference integer at key, defValue if not found.
	 */
	static public Integer getInt(String key, Integer defValue){
		return sharedPrefs().getInt(key, defValue);
	}
	/**
	 * Puts the specified string value at the given key.
	 * 
	 * @param key The key to set.
	 * @param value The string value to set.
	 */
	static public void putString(String key, String value){
		currentEditor().putString(key, value);
	}
	/**
	 * Puts the specified integer value at the given key.
	 * 
	 * @param key The key to set.
	 * @param value The integer value to set.
	 */
	static public void putInt(String key, int value){
		currentEditor().putInt(key, value);
	}
	/**
	 * Removes the value at the specified key.
	 * 
	 * @param key The key to remove.
	 */
	static public void remove(String key){
		currentEditor().remove(key);
	}
	/**
	 * Commits the current transaction editor if any.
	 */
	static public void save(){
		if(smCurrentEditor != null){
			currentEditor().commit();
			smCurrentEditor = null;
		}
	}
	/**
	 * Deletes the current transaction editor without committing.
	 */
	static public void rollback(){
		smCurrentEditor = null;
	}
}
